﻿/*--------------------------------------------------------------------------------------+
|   CreateLabelTest.cpp
|
+--------------------------------------------------------------------------------------*/

#include <Mstn\MdlApi\MdlApi.h>
#include <Mstn\MdlApi\mdl.h>
#include <Mstn\MdlApi\dlogitem.h>
#include <DgnView\DgnElementSetTool.h>
#include <Mstn\ISessionMgr.h>
#include <DgnPlatform\LabelHandler.h>
#include "CreateLabelTest.h"

USING_NAMESPACE_BENTLEY
USING_NAMESPACE_BENTLEY_DGNPLATFORM

Public void startExampleModifyElementTool(WCharCP unparsed);

/*----------------------------------------------------------------------------------+
* The main entry point for the native code MDL application.						    *
* @param        argc     The count of args passed to the application			    *
* @param        argv[]   The array of args passed to this application as char arrays*
+-----------------------------------------------------------------------------------*/
extern "C" DLLEXPORT void MdlMain(int argc, WCharCP argv[])
{
	startExampleModifyElementTool(L"");
}


BentleyStatus lifalib_placeLabel(ElementRefP elmRefIn, char *cellName, double scale, DPoint3d ptOrigin, double offsetX, double offsetY, bool placeLeader, double rotAngleInDregrees = 0.0, UInt32 levelID = 0)
{
	MSElementDescr *pedCell;
	DPoint3d		ptScale, ptTemp;
	wchar_t			wcCellName[MAX_CELLNAME_LENGTH];
	BentleyStatus	rc = SUCCESS;

	EditElementHandle eeh(elmRefIn);

	ptTemp = ptOrigin;
	ptTemp.x -= offsetX;
	ptTemp.y -= offsetY;

	mbstowcs(wcCellName, cellName, MAX_CELLNAME_LENGTH);
	ptScale.x = ptScale.y = ptScale.z = scale;
	if (mdlCell_getElmDscr(&pedCell, NULL, &ptOrigin, &ptScale, true, NULL, NULL, NULL, 0, true, wcCellName, NULL) == SUCCESS)
	{
		ElementHandle ehCell(pedCell, true, true, ISessionMgr::GetActiveDgnModelRefP());
		EditElementHandle labelElm, leaderElm;
		DimensionStylePtr dimStylePtr = DimensionStyle::GetActive();

		// set terminator size
		double arrowSizeSave;
		dimStylePtr->GetDoubleProp(arrowSizeSave, DimStyleProp::DIMSTYLE_PROP_Terminator_Width_DOUBLE);
		dimStylePtr->SetDoubleProp(scale, DimStyleProp::DIMSTYLE_PROP_Terminator_Width_DOUBLE);
		dimStylePtr->SetDoubleProp(scale / 2, DimStyleProp::DIMSTYLE_PROP_Terminator_Height_DOUBLE);

		// get rotationsmatrice for label
		RotMatrix rmLabel = RotMatrix::FromIdentity();
		rmLabel = rmLabel.FromAxisAndRotationAngle(2, rotAngleInDregrees * fc_piover180);
		LabelCellHeaderHandler::StdDPointVector vectorPt;
		vectorPt.push_back(ptTemp);
		vectorPt.push_back(ptOrigin);

		// create label and update reference
		LabelCellHeaderHandler::GetInstance().CreateLabel(labelElm, leaderElm, *dimStylePtr, ehCell, true, rmLabel, *ISessionMgr::GetActiveDgnModelRefP(), vectorPt, !placeLeader);
		LabelCellHeaderHandler::GetInstance().UpdateFieldTarget(labelElm, eeh);

		// restore terminator size
		dimStylePtr->SetDoubleProp(arrowSizeSave, DimStyleProp::DIMSTYLE_PROP_Terminator_Width_DOUBLE);
		dimStylePtr->SetDoubleProp(arrowSizeSave, DimStyleProp::DIMSTYLE_PROP_Terminator_Height_DOUBLE);


		// set grafic group number
		UInt32 ggNum = tcb->graphic;
		ElementPropertiesSetterPtr propSetter = ElementPropertiesSetter::Create();
		propSetter->SetGraphicGroup(labelElm, ggNum);
		propSetter->SetGraphicGroup(leaderElm, ggNum);
		eeh.ReplaceInModel(elmRefIn);
		mdlSystem_updateGraphicGroup();

		// add element(s) to file
		labelElm.AddToModel();
		if (placeLeader)
			leaderElm.AddToModel();

		mdlElmdscr_freeAll(&pedCell);
	}
	else
	{
		wchar_t sMsg[120];
		swprintf(sMsg, L"Error: Unable to find the cell <%hs>", cellName);
		mdlDialog_dmsgsPrint(sMsg);
		rc = ERROR;
	}

	return rc;
}


struct          ExampleModifyElementTool : DgnElementSetTool
{
private:

	bool            m_isDynamics;
	DgnButtonEvent  m_ev;

protected:

	//! @param[in] toolId Set tool command name identifier.
	ExampleModifyElementTool(int toolId) : DgnElementSetTool(toolId) {}

	virtual bool            _DoGroups() override { return false; } // Pick single element, don't include graphic and named group members...
	virtual UsesSelection   _AllowSelection() override { return USES_SS_Check; } // Don't support selection sets...
	virtual UsesFence		_AllowFence() override { return USES_FENCE_Check; }
	virtual bool			_UseActiveFence() override { return true; }
	virtual bool _WantDynamics() override { return false; }
	virtual bool _IsModifyOriginal() override { return false; }
	//virtual void _SetLocateCriteria() override{return true;}

	virtual bool _SetupForModify(DgnButtonEventCR ev, bool isDynamics) override
	{
		m_isDynamics = isDynamics;
		m_ev = ev;

		return true;
	}


	virtual StatusInt _OnElementModify(EditElementHandleR eeh) override
	{
		if (!m_isDynamics)
		{
		}

		return SUCCESS;
	}

#define SCALE mdlModelRef_getUorPerMaster(mdlModelRef_getActive())
	virtual StatusInt _DoOperationForModify(EditElementHandleR eeh) override
	{
		DPoint3d cellOrigin;
		NormalCellHeaderHandler::GetInstance().GetSnapOrigin(eeh, cellOrigin);
		cellOrigin.x += 1.1*SCALE;
		cellOrigin.y += 1.0*SCALE;
		lifalib_placeLabel(eeh.GetElementRef(), "LandInd_alle", 0.1, cellOrigin, 1.1*SCALE, 1.0*SCALE, true, 0.0, eeh.GetElementRef()->GetLevel());

		return SUCCESS;
	}

	virtual bool _OnModifyComplete(DgnButtonEventCR ev)
	{
		//lifalib_dmsg("__OnModifyComplete");
		mdlState_startDefaultCommand();
		return true;
	}

	/*---------------------------------------------------------------------------------**/
	virtual bool _OnPostLocate(HitPathCP path, WStringR cantAcceptReason) override
	{
		if (!DgnElementSetTool::_OnPostLocate(path, cantAcceptReason))
			return false;

		return true;
	}

	/*---------------------------------------------------------------------------------**/
	virtual void _OnRestartTool() override
	{
		InstallNewInstance(GetToolId());
	}

public:
	/*---------------------------------------------------------------------------------**/
	static void InstallNewInstance(int toolId)
	{
		ExampleModifyElementTool* tool = new ExampleModifyElementTool(toolId);

		tool->InstallTool();
	}

}; // ExampleModifyElementTool

/*---------------------------------------------------------------------------------**/
Public void startExampleModifyElementTool(WCharCP unparsed)
{
	ExampleModifyElementTool::InstallNewInstance(0);
}