/*--------------------------------------------------------------------------------------+
|
|   Supplied under applicable software license agreement.
|
|   Copyright (c) 2017 Bentley Systems, Incorporated. All rights reserved.
|
+---------------------------------------------------------------------------------------*/
#pragma once
#pragma once
#include <Bentley/bvector.h>
#include <DgnPlatform/CellHeaderHandler.h>
#include <DgnPlatform/DimensionStyle.h>
BEGIN_BENTLEY_DGNPLATFORM_NAMESPACE
              
/// @addtogroup LabelElements
/// @beginGroup

/*=================================================================================**//**
* A label Cell is-a a sub-type of cell. The label element consists of a leader and a cell
* which are only accessible through this handler. The properties in a label element is 
* controlled by the dimensionstyle defined by the leader element. The label also stores 
* this data internally for leader less labels.
*  @remark Required library : DgnPlatform<ApiNumber>.lib  i.e. DgnPlatform5.lib
* @bsiclass                                                     Bentley Systems
+===============+===============+===============+===============+===============+======*/
struct           LabelCellHeaderHandler : IAnnotationHandler //AnnotationCellHeaderHandler
                                      //   IAnnotationHandler,
{
    DEFINE_T_SUPER(IAnnotationHandler)
    ELEMENTHANDLER_DECLARE_MEMBERS (LabelCellHeaderHandler, DGNPLATFORM_EXPORT)
private:
    virtual void MakeClassAbstract() = 0;
public:
public:

typedef bvector<DPoint3d> StdDPointVector;

//! Creates a new label element.
//! @param    labelElem      OUT the newly created label element.
//! @param    leaderElement OUT The dimension element which will be the label Leader.
//! @param    dimStyle      IN  A style pointer which will provide the label settings.
//! @param    celltemplate  IN  the cell template from which the label to be created from.
//! @param    is3d          IN  flag to control whether to create a 3d element.
//! @param    orientation   IN  orientation of the element.
//! @param    modelRef      IN  destination model of the label.
//! @param    labelLeaderPoints      IN  the points which would define the label leader dimension.
//! @param    isCallOut         IN specify whether we are creating a label without leader.
//! @return   status.       SUCCESS or ERROR  
DGNPLATFORM_EXPORT static BentleyStatus     CreateLabel (EditElementHandleR labelElem, EditElementHandleR leaderElement, DimensionStyleCR dimStyle, ElementHandleCR celltemplate,
                                                            bool is3d, RotMatrixCR orientation, DgnModelRefR modelRef, StdDPointVector const & labelLeaderPoints, bool isCallOut);
//! Adds the label element to a model.
DGNPLATFORM_EXPORT BentleyStatus     AddToModel (EditElementHandleR labelElem, EditElementHandleR leaderElement, DgnModelR dgnCache);

//! Get dimension style associated with a label.
DGNPLATFORM_EXPORT DimensionStylePtr GetLabelDimensionStyle(ElementHandleCR labelElement);

//! Set up the offset association to the target element with the given assoc point.
DGNPLATFORM_EXPORT BentleyStatus    SetupOffsetAssociation (EditElementHandleR labelElement, ElementHandleCR targetElement, AssocPoint const& assoc);

//! Update the unassigned fields inside the cell to obtain values from the target element.
DGNPLATFORM_EXPORT static BentleyStatus   UpdateFieldTarget(EditElementHandleR labelCell, ElementHandleCR newtarget);

}; // LabelCellHeaderHandler


/// @endGroup

END_BENTLEY_DGNPLATFORM_NAMESPACE

